"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.defineRefreshToken = defineRefreshToken;
const const_1 = require("./const");
const assist_1 = require("./assist");
function defineRefreshToken(config) {
    const { refreshRequest, request, isRefresh, refreshRequestCount } = Object.assign(const_1.defineConfig, config);
    let refreshing = false;
    /**
     * 等待刷新请求的映射列表
     */
    let onRefreshReqMap = new Map();
    /**
     * 刷新请求的次数的映射列表
     */
    let onRefreshReqCount = new Map();
    /**
     *
     * @param key
     * @returns
     */
    function _getCurrontRefreshCount(key) {
        if (onRefreshReqCount.has(key)) {
            return onRefreshReqCount.get(key);
        }
        return 0;
    }
    function trackRefreshRequestConfig(config) {
        return new Promise((resolve, reject) => {
            const key = (0, assist_1.getMapKey)(config);
            const cb = () => request(config).then(resolve).catch(reject);
            onRefreshReqMap.set(key, cb);
            if (onRefreshReqCount.has(key)) {
                onRefreshReqCount.set(key, _getCurrontRefreshCount(key) + 1);
            }
            else {
                onRefreshReqCount.set(key, 1);
            }
        });
    }
    function startRefreshServer() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!refreshing) {
                refreshing = true;
                // 请求刷新token
                try {
                    yield refreshRequest();
                }
                catch (err) {
                    window.console.error(err);
                }
                ;
                Promise.resolve().then(() => {
                    refreshing = false;
                });
            }
            const refreshReqList = [...onRefreshReqMap.entries()];
            refreshReqList.forEach((item) => __awaiter(this, void 0, void 0, function* () {
                var _a;
                // 重新执行该request
                try {
                    yield ((_a = item[1]) === null || _a === void 0 ? void 0 : _a.call(item));
                }
                catch (err) {
                    window.console.log(err);
                }
                finally {
                    // 请求完成从map中删除当前请求
                    onRefreshReqMap.delete(item[0]);
                }
            }));
        });
    }
    /**
     *  refreshRequestCount or refreshRequestCount 次数设置为大于0 && isRefresh() === true 开启无感刷新 且判断是否开启刷新token
     * @param config
     * @returns
     */
    function _isRefresh(response) {
        var _a;
        /**
         * config 配置的运行刷新的次数
         */
        const targetRefreshCount = (_a = response.config.refreshRequestCount) !== null && _a !== void 0 ? _a : 0;
        /**
         * 当前已刷新过的历史次数
         */
        const currentRefreshCount = _getCurrontRefreshCount((0, assist_1.getMapKey)(response.config));
        /**
         * 是否已经到最大刷新阀值
         */
        const isBottleneck = currentRefreshCount >= targetRefreshCount && currentRefreshCount >= refreshRequestCount;
        /**
         * 允许refreshToken的次数
         */
        const refreshResolveCount = targetRefreshCount > 0 || refreshRequestCount > 0;
        return !isBottleneck && refreshResolveCount && isRefresh(response);
    }
    /**
     * 删除对应记录
     * @param response
     */
    function _deleteRefreshRequestMap(response) {
        const maps = [onRefreshReqMap, onRefreshReqCount];
        maps.forEach((map) => {
            const key = (0, assist_1.getMapKey)(response.config);
            if (map.has(key)) {
                map.delete(key);
            }
        });
    }
    /**
     * 设置刷新token器
     * @param instance
     * @param plugins
     */
    function refreshTokenSetup(instance = request, plugins) {
        var _a;
        function useRefreshToken(response, err) {
            if (_isRefresh(response)) {
                startRefreshServer();
                window.console.warn(`执行刷新token请求`);
                return trackRefreshRequestConfig(response.config);
            }
            else {
                // 删除当前请求记录数
                _deleteRefreshRequestMap(response);
            }
            if (err) {
                return Promise.reject(err);
            }
            return Promise.resolve(response);
        }
        // 添加响应拦截器
        instance.interceptors.response.use(function (response) {
            return useRefreshToken(response);
        }, (error) => {
            return useRefreshToken(error.response, error);
        });
        // 应用所有插件
        (_a = plugins === null || plugins === void 0 ? void 0 : plugins.forEach) === null || _a === void 0 ? void 0 : _a.call(plugins, p => {
            if (typeof p !== 'function')
                return;
            p(instance);
        });
        return instance;
    }
    return {
        refreshing,
        onRefreshReqMap,
        onRefreshReqCount,
        refreshTokenSetup,
    };
}
