"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defineAbortSignalThrottle = defineAbortSignalThrottle;
const assist_1 = require("./assist");
const const_1 = require("./const");
/**
 * 定义节流请求
 * @returns
 */
function defineAbortSignalThrottle(config = {}) {
    const _defineConfig = Object.assign(const_1.defineConfig, config);
    /**
     * 终止器存储
     */
    const abortControllerMap = new Map();
    /**
     * 最大并发对象次数记录
     */
    const targetMcqMap = new Map();
    function batchMapDelete(key) {
        if (abortControllerMap.has(key)) {
            const controller = abortControllerMap.get(key);
            controller.abort();
            abortControllerMap.delete(key);
        }
        if (targetMcqMap.has(key)) {
            targetMcqMap.delete(key);
        }
    }
    function throttleAbortSignal(config) {
        var _a;
        const key = (0, assist_1.getMapKey)(config);
        const { isBottleneck } = requestCount(key, config);
        if (abortControllerMap.has(key) && isBottleneck) {
            const controller = abortControllerMap.get(key);
            controller.abort();
        }
        const { controller, abort, signal } = (0, assist_1.useAbortSignal)();
        abortControllerMap.set(key, controller);
        const sourceSignal = config === null || config === void 0 ? void 0 : config.signal;
        if (sourceSignal) {
            (_a = sourceSignal === null || sourceSignal === void 0 ? void 0 : sourceSignal.addEventListener) === null || _a === void 0 ? void 0 : _a.call(sourceSignal, 'abort', abort);
        }
        config.signal = signal;
        abortControllerMap.set(key, controller);
        return {
            config,
            abort: () => abort,
        };
    }
    function requestCount(_key, config) {
        let isBottleneck = false;
        if (targetMcqMap.has(_key)) {
            const currentCount = targetMcqMap.get(_key);
            let isMaxConcurrentQuery = currentCount >= _defineConfig.maxConcurrentQueryCount;
            let targetMcqCount = config.maxConcurrentQueryCount;
            if (typeof targetMcqCount === 'number') {
                // 自身有定义则使用自身的 maxConcurrentQueryCount;
                isMaxConcurrentQuery = currentCount >= targetMcqCount;
            }
            if (isMaxConcurrentQuery) {
                window.console.warn(`请求 ${_key} 并发了${currentCount} 已超出并发数 ${_defineConfig.maxConcurrentQueryCount} 已自动终止超出请求 可配置参数maxConcurrentQueryCount调整并发数量`);
                isBottleneck = true;
                targetMcqMap.delete(_key);
            }
            targetMcqMap.set(_key, currentCount + 1);
        }
        else {
            targetMcqMap.set(_key, 1);
        }
        return {
            isBottleneck,
        };
    }
    /**
     * 销毁终止信号
     * @param config
     */
    function destroyAbortSignal(config) {
        const key = (0, assist_1.getMapKey)(config);
        batchMapDelete(key);
    }
    /**
     * 设置终止器
     * @param instance
     * @param plugins
     */
    function abortSignalSetup(instance, plugins) {
        var _a;
        instance.interceptors.request.use(function (config) {
            // throttle 没有设置 获取设置为true 开启节流
            if (config.throttle === void 0 || config.throttle) {
                throttleAbortSignal(config);
            }
            return config;
        });
        // 添加响应拦截器
        instance.interceptors.response.use(function (response) {
            destroyAbortSignal(response.config);
            return Promise.resolve(response);
        }, function (error) {
            return Promise.reject(error);
        });
        // 应用所有插件
        (_a = plugins === null || plugins === void 0 ? void 0 : plugins.forEach) === null || _a === void 0 ? void 0 : _a.call(plugins, p => {
            if (typeof p !== 'function')
                return;
            p(instance);
        });
        return instance;
    }
    return {
        throttleAbortSignal,
        destroyAbortSignal,
        abortSignalSetup,
    };
}
